@@ -24,127 +24,126 @@ from __future__ import division, absolute_import, print_function
 
 import os
 import re
+
+from six import string_types
+
 import numpy as np
 
-try:
-    import lal
-except ImportError:
-    raise ImportError("SWIG wrappings of LAL cannot be imported")
-
-try:
-    import lalpulsar
-except ImportError:
-    raise ImportError("SWIG wrappings of LALPulsar cannot be imported")
-
-try:
-    from six import string_types
-except ImportError:
-    raise ImportError("Could not import six")
-
-try:
-    import astropy
-except ImportError:
-    raise ImportError("Could not import astropy")
-
-
 from astropy import units as u
+
+import lal
+import lalpulsar
+
 # set units of parameters in the PulsarParameters structure
-PPUNITS = {'F':          u.Hz,                     # Hz
-           'P':          u.s,                      # seconds
-           'DIST':       u.m,                      # metres
-           'PX':         u.rad,                    # radians
-           'DM':         u.pc/(u.cm)**3,           # cm^-3 pc
-           'DM1':        u.pc/(u.cm**3*u.yr),      # pc cm^-3 yr^-1
-           'RA':         u.rad,                    # radians
-           'RAJ':        u.rad,                    # radians
-           'DEC':        u.rad,                    # radians
-           'DECJ':       u.rad,                    # radians
-           'PMRA':       u.rad/u.s,                # rad/s
-           'PMDEC':      u.rad/u.s,                # rad/s
-           'ELONG':      u.rad,                    # rad
-           'ELAT':       u.rad,                    # rad
-           'PEPOCH':     u.s,                      # GPS seconds
-           'POSEPOCH':   u.s,                      # GPS seconds
-           'DMEPOCH':    u.s,                      # GPS seconds
-           'GLEP':       u.s,                      # GPS seconds
-           'GLPH':       u.rad,                    # rad
-           'GLF0':       u.Hz,                     # Hz
-           'GLF1':       u.Hz/u.s,                 # Hz/s
-           'GLF2':       u.Hz/u.s**2,              # Hz s^-2
-           'GLF0D':      u.Hz,                     # Hz
-           'GLTD':       u.s,                      # sec
-           'A1':         u.s,                      # light seconds
-           'OM':         u.rad,                    # rad
-           'PB':         u.s,                      # seconds
-           'T0':         u.s,                      # GPS seconds
-           'TASC':       u.s,                      # GPS seconds
-           'EPS1':       u.dimensionless_unscaled,
-           'EPS2':       u.dimensionless_unscaled,
-           'GAMMA':      u.s,                      # seconds
-           'OMDOT':      u.rad/u.s,                # rad/s
-           'XDOT':       u.s/u.s,                  # light seconds/sec
-           'PBDOT':      u.s/u.s,                  # s/s
-           'EDOT':       1./u.s,                   # 1/sec
-           'EPSDOT1':    1./u.s,                   # 1/sec
-           'EPSDOT2':    1./u.s,                   # 1/sec
-           'XPBDOT':     u.s/u.s,                  # s/s
-           'SINI':       u.dimensionless_unscaled,
-           'MTOT':       u.kg,                     # kg
-           'M2':         u.kg,                     # kg
-           'DR':         u.dimensionless_unscaled,
-           'DTHETA':     u.dimensionless_unscaled,
-           'SHAPMAX':    u.dimensionless_unscaled,
-           'A1_2':       u.s,                      # light seconds
-           'A1_3':       u.s,                      # light seconds
-           'OM_2':       u.rad,                    # radians
-           'OM_3':       u.rad,                    # radians
-           'PB_2':       u.s,                      # seconds
-           'PB_3':       u.s,                      # seconds
-           'T0_2':       u.s,                      # GPS seconds
-           'T0_3':       u.s,                      # GPS seconds
-           'FB':         u.Hz,                     # Hz
-           'A0':         u.s,                      # seconds
-           'B0':         u.s,                      # seconds
-           'D_AOP':      1./u.rad,                 # 1/rad
-           'KIN':        u.rad,                    # radians
-           'KOM':        u.rad,                    # radians
-           'WAVE_OM':    u.Hz,                     # Hz
-           'WAVEEPOCH':  u.s,                      # GPS seconds
-           'WAVESIN':    u.s,                      # seconds
-           'WAVECOS':    u.s,                      # seconds
-           'START':      u.s,                      # GPS seconds
-           'FINISH':     u.s,                      # GPS seconds
-           'TRES':       u.s,                      # seconds
-           'H0':         u.dimensionless_unscaled,
-           'APLUS':      u.dimensionless_unscaled,
-           'ACROSS':     u.dimensionless_unscaled,
-           'PHI0':       u.rad,                    # radians
-           'PSI':        u.rad,                    # radians
-           'COSIOTA':    u.dimensionless_unscaled,
-           'IOTA':       u.rad,                    # radians
-           'C22':        u.dimensionless_unscaled,
-           'C21':        u.dimensionless_unscaled,
-           'PHI22':      u.rad,                    # radians
-           'PHI21':      u.rad,                    # radians
-           'CGW':        u.dimensionless_unscaled,
-           'LAMBDA':     u.rad,                    # radians
-           'COSTHETA':   u.dimensionless_unscaled,
-           'THETA':      u.rad,
-           'I21':        u.dimensionless_unscaled,
-           'I31':        u.dimensionless_unscaled,
-           'Q22':        u.kg*u.m**2,              # kg m^2
-           'HPLUS':      u.dimensionless_unscaled,
-           'HCROSS':     u.dimensionless_unscaled,
-           'PSITENSOR':  u.rad,                    # radians
-           'PHI0TENSOR': u.rad,                    # radians
-           'HSCALARB':   u.dimensionless_unscaled,
-           'HSCALARL':   u.dimensionless_unscaled,
-           'PSISCALAR':  u.rad,                    # radians
-           'PHI0SCALAR': u.rad,                    # radians
-           'HVECTORX':   u.dimensionless_unscaled,
-           'HVECTORY':   u.dimensionless_unscaled,
-           'PSIVECTOR':  u.rad,                    # radians
-           'PHI0VECTOR': u.rad                     # radians
+PPUNITS = {'F':            u.Hz,                     # Hz
+           'P':            u.s,                      # seconds
+           'DIST':         u.m,                      # metres
+           'PX':           u.rad,                    # radians
+           'DM':           u.pc/(u.cm)**3,           # cm^-3 pc
+           'DM1':          u.pc/(u.cm**3*u.yr),      # pc cm^-3 yr^-1
+           'RA':           u.rad,                    # radians
+           'RAJ':          u.rad,                    # radians
+           'DEC':          u.rad,                    # radians
+           'DECJ':         u.rad,                    # radians
+           'PMRA':         u.rad/u.s,                # rad/s
+           'PMDEC':        u.rad/u.s,                # rad/s
+           'ELONG':        u.rad,                    # rad
+           'ELAT':         u.rad,                    # rad
+           'PEPOCH':       u.s,                      # GPS seconds
+           'POSEPOCH':     u.s,                      # GPS seconds
+           'DMEPOCH':      u.s,                      # GPS seconds
+           'GLEP':         u.s,                      # GPS seconds
+           'GLPH':         u.rad,                    # rad
+           'GLF0':         u.Hz,                     # Hz
+           'GLF1':         u.Hz/u.s,                 # Hz/s
+           'GLF2':         u.Hz/u.s**2,              # Hz s^-2
+           'GLF0D':        u.Hz,                     # Hz
+           'GLTD':         u.s,                      # sec
+           'A1':           u.s,                      # light seconds
+           'OM':           u.rad,                    # rad
+           'PB':           u.s,                      # seconds
+           'T0':           u.s,                      # GPS seconds
+           'TASC':         u.s,                      # GPS seconds
+           'EPS1':         u.dimensionless_unscaled,
+           'EPS2':         u.dimensionless_unscaled,
+           'GAMMA':        u.s,                      # seconds
+           'OMDOT':        u.rad/u.s,                # rad/s
+           'XDOT':         u.s/u.s,                  # light seconds/sec
+           'PBDOT':        u.s/u.s,                  # s/s
+           'EDOT':         1./u.s,                   # 1/sec
+           'EPSDOT1':      1./u.s,                   # 1/sec
+           'EPSDOT2':      1./u.s,                   # 1/sec
+           'XPBDOT':       u.s/u.s,                  # s/s
+           'SINI':         u.dimensionless_unscaled,
+           'MTOT':         u.kg,                     # kg
+           'M2':           u.kg,                     # kg
+           'DR':           u.dimensionless_unscaled,
+           'DTHETA':       u.dimensionless_unscaled,
+           'SHAPMAX':      u.dimensionless_unscaled,
+           'A1_2':         u.s,                      # light seconds
+           'A1_3':         u.s,                      # light seconds
+           'OM_2':         u.rad,                    # radians
+           'OM_3':         u.rad,                    # radians
+           'PB_2':         u.s,                      # seconds
+           'PB_3':         u.s,                      # seconds
+           'T0_2':         u.s,                      # GPS seconds
+           'T0_3':         u.s,                      # GPS seconds
+           'FB':           u.Hz,                     # Hz
+           'A0':           u.s,                      # seconds
+           'B0':           u.s,                      # seconds
+           'D_AOP':        1./u.rad,                 # 1/rad
+           'KIN':          u.rad,                    # radians
+           'KOM':          u.rad,                    # radians
+           'WAVE_OM':      u.Hz,                     # Hz
+           'WAVEEPOCH':    u.s,                      # GPS seconds
+           'WAVESIN':      u.s,                      # seconds
+           'WAVECOS':      u.s,                      # seconds
+           'START':        u.s,                      # GPS seconds
+           'FINISH':       u.s,                      # GPS seconds
+           'TRES':         u.s,                      # seconds
+           'H0':           u.dimensionless_unscaled,
+           'APLUS':        u.dimensionless_unscaled,
+           'ACROSS':       u.dimensionless_unscaled,
+           'PHI0':         u.rad,                    # radians
+           'PSI':          u.rad,                    # radians
+           'COSIOTA':      u.dimensionless_unscaled,
+           'IOTA':         u.rad,                    # radians
+           'C22':          u.dimensionless_unscaled,
+           'C21':          u.dimensionless_unscaled,
+           'PHI22':        u.rad,                    # radians
+           'PHI21':        u.rad,                    # radians
+           'CGW':          u.dimensionless_unscaled,
+           'LAMBDA':       u.rad,                    # radians
+           'COSTHETA':     u.dimensionless_unscaled,
+           'THETA':        u.rad,
+           'I21':          u.dimensionless_unscaled,
+           'I31':          u.dimensionless_unscaled,
+           'Q22':          u.kg*u.m**2,              # kg m^2
+           'H0_F':         u.dimensionless_unscaled,
+           'HPLUS':        u.dimensionless_unscaled,
+           'HCROSS':       u.dimensionless_unscaled,
+           'PSITENSOR':    u.rad,                    # radians
+           'PHI0TENSOR':   u.rad,                    # radians
+           'HSCALARB':     u.dimensionless_unscaled,
+           'HSCALARL':     u.dimensionless_unscaled,
+           'PSISCALAR':    u.rad,                    # radians
+           'PHI0SCALAR':   u.rad,                    # radians
+           'HVECTORX':     u.dimensionless_unscaled,
+           'HVECTORY':     u.dimensionless_unscaled,
+           'PSIVECTOR':    u.rad,                    # radians
+           'PHI0VECTOR':   u.rad,                    # radians
+           'HPLUS_F':      u.dimensionless_unscaled,
+           'HCROSS_F':     u.dimensionless_unscaled,
+           'PSITENSOR_F':  u.rad,                    # radians
+           'PHI0TENSOR_F': u.rad,                    # radians
+           'HSCALARB_F':   u.dimensionless_unscaled,
+           'HSCALARL_F':   u.dimensionless_unscaled,
+           'PSISCALAR_F':  u.rad,                    # radians
+           'PHI0SCALAR_F': u.rad,                    # radians
+           'HVECTORX_F':   u.dimensionless_unscaled,
+           'HVECTORY_F':   u.dimensionless_unscaled,
+           'PSIVECTOR_F':  u.rad,                    # radians
+           'PHI0VECTOR_F': u.rad                     # radians
           }
 
 # set units of parameters in a TEMPO-style parameter file if different from above
@@ -162,6 +161,7 @@ TEMPOUNITS = {'DIST':      u.kpc,                  # kpc
               'POSEPOCH':  u.d,                    # MJD(TT) (day)
               'DMEPOCH':   u.d,                    # MJD(TT) (day)
               'GLEP':      u.d,                    # MJD(TT) (day)
+              'GLTD':      u.d,                    # days
               'OM':        u.deg,                  # degs
               'PB':        u.d,                    # day
               'T0':        u.d,                    # MJD(TT) (day)
@@ -259,7 +259,12 @@ class PulsarParametersPy(object):
                 raise ValueError("Expected 'lalpulsar.PulsarParameters' type, string, or None")
 
     def __len__(self):
-        return length
+        _ = self.keys()  # length is counted in the keys() method
+
+        return self.length
+
+    def __str__(self):
+        return self.pp_to_str()
 
     def __getitem__(self, key):
         """
@@ -276,12 +281,15 @@ class PulsarParametersPy(object):
             geterr = True
             tkey = key[:-4] # get the actual parameter key name
 
-        # check if the key is asking for an individal parameter from a vector parameter
-        # (e.g. 'F0' gets the first value from the 'F' vector)
+        # check if the key is asking for an individual parameter from a vector parameter
+        # (e.g. 'F0' gets the first value from the 'F' vector.
+        # NOTE: this is problematic for glitch parameters, e.g., GLF0, which could provide
+        # values to multiple glitches, so this cannot be used to get individual glitch
+        # parameters).
         sname = re.sub(r'_\d', '', tkey) if '_' in tkey else re.sub(r'\d', '', tkey)
         sidx = None
         indkey = None
-        if sname != tkey:
+        if sname != tkey and tkey[0:2] != "GL":
             # check additional index is an integer
             try:
                 sidx = int(tkey.split('_')[-1]) if '_' in tkey else int(tkey[len(sname):])
@@ -293,6 +301,11 @@ class PulsarParametersPy(object):
                 indkey = tkey # key with index
                 tkey = sname
 
+            # check if parameter key is present otherwise switch back to original name
+            # (needed for, e.g., 'H0', 'PHI0', ...)
+            if not lalpulsar.PulsarCheckParam(self._pulsarparameters, tkey):
+                tkey = indkey
+
         # check if parameter given by the key is present
         if not lalpulsar.PulsarCheckParam(self._pulsarparameters, tkey):
             return None
@@ -388,7 +401,7 @@ class PulsarParametersPy(object):
         if isinstance(value, np.ndarray) or isinstance(value, list):
             cvalue = []
             bunit = ppunit
-            for i, v in enumerate(value):
+            for v in value:
                 cvalue.append(v*ppunit)
                 if uname in ['F', 'FB', 'P']: # frequency/period values
                     ppunit *= bunit # increment unit (e.g. Hz -> Hz/s, Hz/s -> Hz/s^2)
@@ -665,35 +678,41 @@ class PulsarParametersPy(object):
 
         fitflag = lalpulsar.PulsarGetParamFitFlagAsVector(self._pulsarparameters, name)
 
-        if len(fitflag.data) > 1:
+        if len(fitflag.data) > 1 or isinstance(self[name.upper()], (list, np.ndarray)):
             return fitflag.data
         else:
             return fitflag.data[0]
 
-    def pp_to_par(self, filename, precision=19):
+    def pp_to_str(self, precision=19):
         """
-        Output the PulsarParameter structure to a `.par` file.
+        Convert the PulsarParameter structure to a string in the format of a
+        TEMPO-style `.par` file.
 
         Args:
-            filename (str): the path to the output file
             precision (int): the number of decimal places for an output value
+
+        Returns:
+            str: the contents in TEMPO-format
         """
 
-        try:
-            fp = open(filename, 'w')
-        except IOError:
-            raise IOError("Could not open file '{}' for writing".format(filename))
-
         # output string format (set so that values should line up)
-        mkl = max([len(kn) for kn in self.keys()])+2 # max key length for output alignment
-        vlb = precision+10 # allow extra space for minus sign/exponents
-        outputstr = '{{name: <{0}}}{{value: <{1}}}{{fitflag}}\t{{error}}'.format(mkl, vlb)
+        mkl = max([len(kn) for kn in self.keys()]) + 2  # max key length for output alignment
+        vlb = precision + 10  # allow extra space for minus sign/exponents
+        outputstr = "{{name: <{0}}}{{value: <{1}}}{{fitflag}}\t{{error}}".format(mkl, vlb)
+
+        parstr = ""
+
+        parsedwaves = False
 
         # get names of parameters in file
         for item in self.items():
             key = item[0]
             value = item[1]
 
+            if key in ["WAVESIN", "WAVECOS"] and parsedwaves:
+                # already added the FITWAVES values
+                continue
+
             # get error
             evalue = self.get_error(key)
 
@@ -724,38 +743,72 @@ class PulsarParametersPy(object):
             if evalue is not None:
                 fitflag = self.get_fitflag(key)
 
-            oevalue = ''  # default output error value
-            ofitflag = ' ' # default output fit flag value (a single space for alignment purposes)
+            oevalue = ""  # default output error value
+            ofitflag = " "  # default output fit flag value (a single space for alignment purposes)
             if isinstance(tvalue, list) or isinstance(tvalue, np.ndarray):
                 idxoffset = 0
-                idxsep = ''
-                if key in ['WAVESIN', 'WAVECOS', 'GLEP', 'GLPH', 'GLF0', 'GLF0D', 'GLTD']:
+                idxsep = ""
+                if key in ['WAVESIN', 'WAVECOS', 'GLEP', 'GLPH', 'GLF0', 'GLF1', 'GLF2', 'GLF0D', 'GLTD']:
                     # the TEMPO variable name for these parameter start with an index a 1
                     idxoffset = 1
 
-                    if key[:2] == 'GL':
-                        # glitch parameters have an '_' seperating the name from the index
-                        idxsep = '_'
+                    if key[:2] == "GL":
+                        # glitch parameters have an "_" seperating the name from the index
+                        idxsep = "_"
 
-                for tv, te, tf in zip(tvalue, tevalue, fitflag):
-                    precstr = '{{0:.{}f}}'.format(precision) # print out float
-                    if tv < 1e-6 or tv > 1e6:
-                        # print out float in scientific notation
-                        precstr = '{{0:.{}e}}'.format(precision)
+                if key in ["WAVESIN", "WAVECOS"]:
+                    # do things differently for FITWAVES parameters
+                    parsedwaves = True
 
-                    precstre = '{{0:.{}f}}'.format(precision) # print out float
-                    if te < 1e-6 or te > 1e6:
-                        # print out float in scientific notation
-                        precstre = '{{0:.{}e}}'.format(precision)
+                    if key == "WAVESIN":
+                        wavesin = tvalue
+                        wavecos = self["WAVECOS"]
+                    else:
+                        wavesin = self["WAVESIN"]
+                        wavecos = tvalue
 
-                    outputdic = {}
-                    outputdic['name'] = '{}{}{}'.format(key, idxsep, idxoffset)
-                    idxoffset += 1
-                    outputdic['value'] = precstr.format(tv)
-                    outputdic['fitflag'] = '1' if tf == 1 else ''
-                    outputdic['error'] = precstre.format(te) if te != 0. else ''
+                    for ws, wc in zip(wavesin, wavecos):
+                        precstrs = '{{0:.{}f}}'.format(precision)  # print out float
+                        if ws < 1e-6 or ws > 1e6:
+                            # print out float in scientific notation
+                            precstrs = '{{0:.{}e}}'.format(precision)
 
-                    fp.write(outputstr.format(**outputdic).strip()+'\n')
+                        precstrc = '{{0:.{}f}}'.format(precision)  # print out float
+                        if wc < 1e-6 or wc > 1e6:
+                            # print out float in scientific notation
+                            precstrc = '{{0:.{}e}}'.format(precision)
+
+                        outputdic = {}
+                        outputdic["name"] = "WAVE{}{}".format(idxsep, idxoffset)
+                        idxoffset += 1
+                        outputdic["value"] = "{}\t{}".format(
+                            precstrs.format(ws),
+                            precstrc.format(wc)
+                        )
+                        outputdic["fitflag"] = ""
+                        outputdic["error"] = ""
+
+                        parstr += outputstr.format(**outputdic).strip() + "\n"
+                else:
+                    for tv, te, tf in zip(tvalue, tevalue, fitflag):
+                        precstr = '{{0:.{}f}}'.format(precision) # print out float
+                        if tv < 1e-6 or tv > 1e6:
+                            # print out float in scientific notation
+                            precstr = '{{0:.{}e}}'.format(precision)
+
+                        precstre = '{{0:.{}f}}'.format(precision) # print out float
+                        if te < 1e-6 or te > 1e6:
+                            # print out float in scientific notation
+                            precstre = '{{0:.{}e}}'.format(precision)
+
+                        outputdic = {}
+                        outputdic['name'] = '{}{}{}'.format(key, idxsep, idxoffset)
+                        idxoffset += 1
+                        outputdic['value'] = precstr.format(tv)
+                        outputdic['fitflag'] = '1' if tf == 1 else ''
+                        outputdic['error'] = precstre.format(te) if te != 0. else ''
+
+                        parstr += outputstr.format(**outputdic).strip() + "\n"
             else:
                 if isinstance(tvalue, float) or key in ['RA', 'RAJ', 'DEC', 'DECJ']:
                     if isinstance(tvalue, float):
@@ -790,8 +843,25 @@ class PulsarParametersPy(object):
                 outputdic['fitflag'] = ofitflag
                 outputdic['error'] = oevalue
 
-                fp.write(outputstr.format(**outputdic).strip()+'\n')
+                parstr += outputstr.format(**outputdic).strip() + "\n"
 
+        return parstr
+
+    def pp_to_par(self, filename, precision=19):
+        """
+        Output the PulsarParameter structure to a `.par` file.
+
+        Args:
+            filename (str): the path to the output file
+            precision (int): the number of decimal places for an output value
+        """
+
+        try:
+            fp = open(filename, 'w')
+        except IOError:
+            raise IOError("Could not open file '{}' for writing".format(filename))
+
+        fp.write(self.pp_to_str(precision=precision))
         fp.close()
 
     def PulsarParameters(self):
@@ -800,3 +870,13 @@ class PulsarParametersPy(object):
         """
 
         return self._pulsarparameters
+
+    def __deepcopy__(self, memo):
+        """
+        Create a copy of the parameters.
+        """
+
+        newpar = PulsarParametersPy()
+        memo[id(self)] = newpar
+        lalpulsar.PulsarCopyParams(self.PulsarParameters(), newpar.PulsarParameters())
+        return newpar
