@@ -22,5 +22,16 @@ del a
 lal.CheckMemoryLeaks()
 print("PASSED object parent tracking")
 
+# check multi-vector element assignment
+print("checking multi-vector element assignment ...")
+mts = lalpulsar.CreateMultiLIGOTimeGPSVector(2)
+ts0 = lalpulsar.CreateTimestampVector(3)
+mts.data[0] = ts0
+lal.swig_set_nasty_error_handlers()
+del mts
+del ts0
+lal.swig_set_nice_error_handlers()
+print("PASSED multi-vector element assignment")
+
 # passed all tests!
 print("PASSED all tests")
