@@ -24,7 +24,7 @@ typedef struct tagParamData{
   const CHAR *fitFlag; /* add a TEMPO-style fitting flag to some parameters */
 }ParamData;
 
-#define NUMPARS 58
+#define NUMPARS 86
 
 /* setup a collection of allowed .par file parameters */
 ParamData p[NUMPARS] =
@@ -94,6 +94,36 @@ ParamData p[NUMPARS] =
   { "PSI",      "-0.12300",     "-0.12300",         "0.01230",      "0.01230"    ,  " " },
   { "APLUS",    "1.23000e-22",  "1.23000e-22",      "1.23000D-23",  "1.23000e-23",  " " }, /* input exponent as D */
   { "ACROSS",   "1.23000e-22",  "1.23000e-22",      "1.23000D-23",  "1.23000e-23",  " " }, /* input exponent as D */
+  /* waveform parameters */
+  { "C21",      "7.83000e-22",  "7.83000e-22",      "4.65000E-23",  "4.65000e-23",  " " },
+  { "C22",      "3.65000E-23",  "3.65000e-23",      "6.54000e-25",  "6.54000e-25",  " " },
+  { "PHI21",    "0.3",          "0.30000",          "0.12",         "0.12000",      " " },
+  { "PHI22",    "2.37281",      "2.37281",          "0.0011",       "0.00110",      " " },
+  /* non-GR parameters */
+  { "HPLUS",        "5.40000e-24",  "5.40000e-24",      "1.26000e-26",  "1.26000e-26",  " " },
+  { "HPLUS_F",      "6.20000e-24",  "6.20000e-24",      "5.28000e-26",  "5.28000e-26",  " " },
+  { "HCROSS",       "7.40000e-24",  "7.40000e-24",      "1.26000e-26",  "1.26000e-26",  " " },
+  { "HCROSS_F",     "8.20000e-24",  "8.20000e-24",      "5.28000e-26",  "5.28000e-26",  " " },
+  { "HVECTORX",     "9.40000e-24",  "9.40000e-24",      "1.26000e-26",  "1.26000e-26",  " " },
+  { "HVECTORX_F",   "1.20000e-24",  "1.20000e-24",      "5.28000e-26",  "5.28000e-26",  " " },
+  { "HVECTORY",     "2.40000e-24",  "2.40000e-24",      "1.26000e-26",  "1.26000e-26",  " " },
+  { "HVECTORY_F",   "3.20000E-24",  "3.20000e-24",      "5.28000e-26",  "5.28000e-26",  " " },
+  { "HSCALARL",     "4.40000D-24",  "4.40000e-24",      "1.26000e-26",  "1.26000e-26",  " " },
+  { "HSCALARL_F",   "5.20000e-24",  "5.20000e-24",      "5.28000e-26",  "5.28000e-26",  " " },
+  { "HSCALARB",     "6.40000E-24",  "6.40000e-24",      "1.26000e-26",  "1.26000e-26",  " " },
+  { "HSCALARB_F",   "7.20000E-24",  "7.20000e-24",      "5.28000e-26",  "5.28000e-26",  " " },
+  { "PSITENSOR",    "1.46252",      "1.46252",          "0.91",         "0.91000",      " " },
+  { "PSITENSOR_F",  "0.3",          "0.30000",          "0.02",         "0.02000",      " " },
+  { "PHI0TENSOR",   "2.65",         "2.65000",          "0.03",         "0.03000",      " " },
+  { "PHI0TENSOR_F", "2.1201",       "2.12010",          "0.04",         "0.04000",      " " },
+  { "PSIVECTOR",    "0.00230",      "0.00230",          "0.05",         "0.05000",      " " },
+  { "PSIVECTOR_F",  "0.786",        "0.78600",          "0.06",         "0.06000",      " " },
+  { "PHI0VECTOR",   "0.10230",      "0.10230",          "0.07",         "0.07000",      " " },
+  { "PHI0VECTOR_F", "0.116",        "0.11600",          "0.08",         "0.08000",      " " },
+  { "PSISCALAR",    "1.00430",      "1.00430",          "0.09",         "0.09000",      " " },
+  { "PSISCALAR_F",  "0.286",        "0.28600",          "0.10",         "0.10000",      " " },
+  { "PHI0SCALAR",   "2.10230",      "2.10230",          "0.11",         "0.11000",      " " },
+  { "PHI0SCALAR_F", "0.476",        "0.47600",          "0.12",         "0.12000",      " " },
 };
 
 
